###############################################################################
# last updated 29 June 2012
#
#  Construct the dataset for the project
#
#  This program builds & outputs a list of 
#     1.  [f1] all 1fx period schedules in the HFD
#     2.  [f5] all 5fx period schedules constructed from (1)
#     3.  [HFD.BASIS] SVD components 1..5 for (1)  
#     4.  [IDB] the collection of 5fx schedules from the US Census Intl Data Base (Schmertmann 2003)
#     5.  [IDB1] the single-year split version of (4), using the Quad Spline coefs from Schmertmann 2003: File @
#     6.  [BIG.BASIS] SVD components 1..5 for [{every 5th year of (1)}+(5)] 
#     7.  [f1.single] a version of f1 in which only rates that came
#          from single-year original data are included. All cells split from 
#          grouped data have NA values.
#     8.  [f1.split] a matrix same dimension as f1, with TRUE/FALSE for whether
#          this rate was split from a group or open interval
#     9.  [hfd.split] a function to split 5fx -> 1fx values for x=12...54 using the HFD protocol
#    10.  [fine.hfd.split] a function to split 5fx -> fine-gridded fx values using the 
#            HFD protocol. This version is for some special calculations and comparisons.
###############################################################################

rm(list=ls())
memory.limit(4000)

library(signal)

WRITE.DATA = TRUE               # save output to a dput file?
outfile    = "fert.data.dput"   # dput file name

DATA = list()

graphics.off()
windows(record=T)
op = par( no.readonly=T)   # save plotting parameters for later reset

## read main HFD data on age-specific rates
RR = read.table("./Original Data/asfrRR.txt",skip=2,as.is=T, header=T)

## convert ages to numbers (get rid of 12- and 55+)
RR$Age = 12:55

## which ages do we want to use
age    = 12:54
A      = length(age)

rates = sched = list()

for (k in unique(RR$Code)) {
  sel        = (RR$Code==k) & (RR$Age %in% age)
  rates[[k]] = tapply( RR$ASFR[sel], list( RR$Age[sel], RR$Year[sel]), sum) 
  sched[[k]] = paste(k,dimnames(rates[[k]])[[2]],sep="") 
}

big = matrix( unlist(rates), nrow=A, dimnames=list(age,unlist(sched)))

DATA$f1 = big

###########################################################

## construct f5 data (9 age groups rather than usual 7)
agroup = list( 10:14, 15:19, 20:24, 25:29,
               30:34, 35:39, 40:44, 45:49, 50:55)

alo = sapply( agroup, min)

C = matrix( 0, length(agroup), A, dimnames=list(alo, age) )

for (i in 1:nrow(C)) {
  sel = age %in% agroup[[i]]
  nsel = sum(sel)
  C[i,sel] = 1/nsel
}

DATA$f5 = C %*% DATA$f1
###########################################################

## SVD decomposition of HFD schedules
ss = svd(big)
X  = ss$u[,1:5]
for (j in 1:ncol(X)) { if (sum(X[,j]) < 0) X[,j] = -X[,j] }

matplot(age,X,type="o",lty=1,lwd=2, ylim=c(-.5,.5), main="SVD of HFD schedules")
abline(h=0)
abline(v=seq(10,60,5),lty=2,col="grey")

DATA$HFD.BASIS = X

#########################################################

IDB = read.csv(file="./Original Data/IDB 5fx.csv", header=T)

colz = paste("f",seq(15,45,5),sep="")

tmp = matrix(unlist(IDB[,colz]), ncol=length(colz)) / 1000
dimnames(tmp)=list(IDB$schedule,colz)

DATA$IDB = tmp


#######################################################

## A X #IDB countries, placeholder matrix of zeroes
fhatIDB = matrix(0, A, nrow(IDB), dimnames=list(age,dimnames(DATA$IDB)[[1]]))

## read the QS single-year fits to the IDB five-year data (covers ages 10..49)
tmp = as.matrix( read.csv("./Original Data/IDB 1fx values.csv", header=T, row.names=1))
dimnames(tmp)[[2]] = 10:49

fhatIDB[paste(12:49), ] = t(tmp)[paste(12:49),]   # fhatIDB cells at ages 50+ are zero

DATA$IDB1 = fhatIDB

####################################################################

## SVD basis using both HFD and split IDB schedules
##  Including IDB adds information about African and Asian schedule
##  shapes that is otherwise missing
##
##  to avoid too much overcounting of highly correlated schedules,
##   this procedures uses every 5th year in the HFD -- eg, USA schedules
##   are avail from 1933-2007, so we use 2007, 2002, 1997, ... 1937

## figure out which columns of "big" to keep

lastcol = cumsum( sapply(sched,length)) ## col numbers of last column for each country

## indices selected columns for first country (AUT)
ix = seq(lastcol[1],1,-5)

## indices selected columns for other countries (BGR...USA)
for (j in 2:length(lastcol)) ix = c(ix, seq(lastcol[j], lastcol[j-1]+1, -5))

## sort just for neatness
ix = sort(ix)

ss = svd(cbind(big[,ix],fhatIDB))
X  = ss$u[,1:5]
for (j in 1:ncol(X)) { if (sum(X[,j]) < 0) X[,j] = -X[,j] }

matplot(age,X,type="o",lty=1,lwd=2, main="SVD of HFD+IDB", ylim=c(-.5,.5))
abline(h=0)
abline(v=seq(10,60,5),lty=2,col="grey")

DATA$BIG.BASIS = X

####### IDENTIFY THE SUBSET OF f1 THAT COMES FROM SINGLE-YEAR DATA ####################
#----------------------------------------
# read the list of which country-year-age 
#  combinations have single-year data
#
#  there are some annoying problems here
#  because the HFD data in "info" are a 
#  newer vintage than those in "D". In 
#  particular, "info" has some extra countries
#  (e.g., ESP) and some extra years for 
#  the countries in "D" (e.g., AUT2010)
#----------------------------------------

##  HFD-supplied routine to process XXXbirths.txt file
##  The important part here is that it will allow identification
##   of the level of aggregation (1-year or 5-year groups) in the
##   original data for each [country x year] combination

################################################################################
##
## Subversion file information - please do not edit this block
## $Rev: 593 $
## $Date: 2010-09-10 11:10:31 +0200 (Fri, 10 Sep 2010) $
## $Author: nash $
## $Id: indbLoad.R 593 2010-09-10 09:10:31Z nash $
##
################################################################################

indbLoad <-
function(fname, LDB=TRUE) {
#loads and translets into internal fomat births data (XXXbirths.txt) 

 indb=read.csv(fname,header=TRUE,colClasses = "character",strip.white=TRUE,na.strings="");
 indb$LDB=as.numeric(indb$LDB);
 if (LDB) indb=indb[indb$LDB==1,];
 indb$Year[indb$Year == "."] <- NA
 indb$Year=as.numeric(indb$Year);

 indb$YearReg[indb$YearReg == "."] <- NA
 indb$YearReg=as.numeric(indb$YearReg);
 indb$Year[is.na(indb$Year)] <- indb$YearReg[is.na(indb$Year)]
 indb$Age[indb$Age=='TOT']=300;
 indb$Age[indb$Age=='UNK']=-1;
 indb$Age=as.numeric(indb$Age);

 indb$AgeInt[indb$AgeInt=='+']=100;
 indb$AgeInt[indb$AgeInt=='-']=-100;
 indb$AgeInt[indb$AgeInt=='.']=NA;
 indb$AgeInt=as.numeric(indb$AgeInt);
 
 indb$Births=as.numeric(indb$Births);
# indb$Births[is.na(indb$Births)]=0;

 indb$BirthOrder[indb$BirthOrder=='TOT']=0;
 indb$BirthOrder[indb$BirthOrder=='UNK']=-1;
 indb$BirthOrder[indb$BirthOrder=='.']=-1;
 indb$BirthOrder=as.numeric(indb$BirthOrder);

 return(indb)
}


#############################################################

pop = c("AUT","BGR","BLR","CAN","CHE","CZE",
        "DEUTE","DEUTNP","DEUTW","ESP","EST","FIN",    
        "FRATNP","GBR_NIR","GBR_SCO","GBRTENW",
        "HUN","IRL","LTU","NLD","PRT","RUS","SVK","SVN",    
        "SWE","UKR","USA")   

## which ages do we want to use
age    = 12:54
A      = length(age)

info = list()

for (p in pop) {

# figure out which (age x year) cells have 1-year data  
  dta = indbLoad(paste("./Original Data/",p,"births.txt",sep=""))
  ok = ( (dta$BirthOrder==0) & (dta$Age %in% 12:54) )
  
## counts of age x year combinations with at least 1 single-year data  
   z = table( dta$Age[ok], dta$Year[ok], dta$AgeInt[ok]==1) [,,"TRUE"]

  ## shell containing same columns and ALL ages
  tmp = matrix(0, A, ncol(z), dimnames=list(age, dimnames(z)[[2]]))
  tmp[dimnames(z)[[1]],] = 1*(z>0)

  info[[p]] = tmp
  
# draw a plot  
  year = as.numeric( dimnames(info[[p]])[[2]] )
  plot( 0, xlim=range(year), ylim=c(12,54), type="n", main=paste(p,"Single-Year Data"))
  abline(h=seq(10,60,5), v=seq(1800,2100,10),lty=2)
  for (y in year) {
      ok  =  (12:54)[ info[[p]][,paste(y)] > 0 ] + .50
      points( rep(y,length(ok)), ok, pch=16, col=grey(.60))
  }
}


f1 = DATA$f1

## try to create a matrix of identical size to D$f1, with NA or 1 values in for each
##   cell, depending on whether the original data was single-year (=1) or not (=NA)

## add country names to column labels

for (k in names(info)) {
  oldlab = dimnames(info[[k]])[[2]]
  newlab = paste(k,oldlab,sep="")
  dimnames(info[[k]])[[2]] = newlab
}

colnames = unlist( lapply(info, function(x) dimnames(x)[[2]]))

big.info = matrix( unlist(info), nrow=A, dimnames=list(age, colnames))
keep     = intersect( dimnames(f1)[[2]], dimnames(big.info)[[2]])

f1.single = f1[,keep]

grouped = which( big.info[,keep] == 0, arr.ind=TRUE)

f1.single[grouped] = NA

DATA$f1.single = f1.single

#########################################################
## matrix same dimension as f1, with TRUE/FALSE for whether
##   this rate was split from a group or open interval
#########################################################

split = matrix(FALSE, nrow(f1), ncol(f1), dimnames=dimnames(f1))
split[grouped] = TRUE

DATA$f1.split = split

#########################################################
## function to split a vector of rates into single years over 12:54
##   if input length is 7 then age groups are 15-19...45-49
##   if input length is 9 then age groups are 10-14...50-54
#########################################################

hfd.split = function(y, LO=20, HI=12) {
  
  if (length(y)==7)  { x = seq(15,45,5) }
  if (length(y)==9)  { x = seq(10,50,5) }
    
  f5 = y
  
  cumF        = c(0, 5*cumsum(f5))
  x           = c(x,max(x)+5)
  names(cumF) = x
  
  maxF = max(cumF)
  
  Y = log( cumF / (maxF - cumF))
  Y[Y < -LO] = -LO
  Y[Y > +HI] = +HI
  
  xval = min(x):max(x)
  
  Yhat = interp1( x, Y, xi=xval, method="pchip")
  Fhat = maxF * exp(Yhat)/(1+exp(Yhat))
  
  f1hat = diff(Fhat)
  names(f1hat) = xval[-length(xval)]
  
  ## fill with NA for 12..min(x) and max(x)...54
  
  xx        = 12:54
  yy        = NA*xx
  names(yy) = xx
  
  ix             = match(xx, names(f1hat))  # which indices of yy are not NA
  yy[paste(xx)]  = f1hat[ix]
  
  return( round(yy,5)  )
    
} #hfd.split

fine.hfd.split = function (y, 
                     LO = 20, HI = 12,  delta=1) 
{
  if (length(y) == 7) {
    x = seq(15, 45, 5)
  }
  if (length(y) == 9) {
    x = seq(10, 50, 5)
  }
  f5 = y
  cumF = c(0, 5 * cumsum(f5))
  x = c(x, max(x) + 5)
  names(cumF) = x
  maxF = max(cumF)
  Y = log(cumF/(maxF - cumF))
  Y[Y < -LO] = -LO
  Y[Y > +HI] = +HI
  xval = seq(min(x),max(x), delta)
  Yhat = interp1(x, Y, xi = xval, method = "pchip")
  Fhat = maxF * exp(Yhat)/(1 + exp(Yhat))
  f1hat = diff(Fhat)/delta
  return(list(x=xval[-length(xval)]+delta/2, 
              y=f1hat,
              indata=y,
              LO=LO,
              HI=HI,
              delta=delta))
}

DATA$hfd.split      = hfd.split
DATA$fine.hfd.split = fine.hfd.split


#---------------------------------------------------

if (WRITE.DATA) dput( DATA, file=outfile)

