##############################################################################
# Construct single-year estimates of HFD and IDB schedules over ages 12..54
#    from five-year average data.  
#
#   There are both unadjusted and adjusted versions of the
#   Beers and CS schedules;  adjusted means that negative values have been
#   purged. 
#
#  the output is:
#     list(
#          HFDsplit = list(HFD,IDB) ,
#          Beers    = list(HFD = list(unadj, adj),
#                          IDB = list(unadj, adj)),
#          CS       = list(HFD = list(unadj, adj),
#                          IDB = list(unadj, adj)),
#         )
# 
##############################################################################

memory.limit(3500)
graphics.off()
windows(record=T)

rm(list=ls())

library(signal)   # for interp1 function used by HFD splitting protocol

OUT     = list()   # output will go here
outfile = "fert.estimates.dput"

#----------------------------------------
# read prepared f5,f1,SVD,NMF... data
#----------------------------------------
D = dget("fert.data.dput")
for (k in names(D)) assign(k, D[[k]])

age = 12:54
A   = length(age)


##############################################################################
# HFD splitting protocol first 
##############################################################################

tmp                = apply(f5, 2, function(sched) hfd.split(sched) )
dimnames(tmp)[[1]] = age
OUT$HFDsplit$HFD   = round( tmp, 4)

tmp                = apply(IDB, 1, function(sched) hfd.split(sched, LO=8) )
dimnames(tmp)[[1]] = age
OUT$HFDsplit$IDB   = round( tmp, 4)


##############################################################################
# Beers method next 
##############################################################################


### get Beers constants
KBeers = 5 * as.matrix( read.csv("./Original Data/Beers matrix.csv", header=T, row.names=1) )

## Liu et al. (2011) function to correct zeroes in left tail
##   fhat is a 40x1 vector of fertility rates from Beers
fixLeft = function(fhat) {
   ix = 15:24-14 # range of indices to check for negatives
   tmp = as.vector(fhat)
   neg = tmp[ix]<0
   if (any(neg)) {
     n = max(which(neg)) 
     rho   = tmp[n+1]/tmp[n+2]  # ratio
     tmp[1:n] = rho^(n:1) * tmp[n+1]
   } 
   return(tmp)
}

## Liu et al. (2011) function to correct zeroes in right tail
##   fhat is a 40x1 vector of fertility rates from Beers
fixRight = function(fhat) {
   ix = 30:54 - 14 # range to check for negatives
   tmp = as.vector(fhat)
   neg = tmp[ix]<0
   if (any(neg)) {
     n = 15+min(which(neg)) 
     rho   = tmp[n-1]/tmp[n-2]  # ratio
     tmp[n:40] = rho^(seq(1,40-n+1)) * tmp[n-1]
   } 
   return(tmp)
}

# Liu et al. (2011): keep proportions within each 5-year group, but re-adjust so 
# that sums match 5-year totals

rescale = function(f1,f5) {
  if (length(f5)==8) {
    mult = (5*f5) / tapply(f1, cut(15:54,breaks=seq(15,55,5),right=F), sum)
    mult = rep(mult, rep(5,8) )
  }
  return(mult * f1[1:length(mult)])
}


## Beers fit for an 8x1 vector of rates for ages 15-19...50-54
##            or  a 7x1 vector of rates for ages 15-19...45-49 (just drop last col of expansion matrix K)
##    if allow.neg=FALSE then use the procedures to clean up negative values

fit.Beers = function(f5, allow.neg=FALSE) {
  
   estim   = KBeers[,1:length(f5)] %*% f5
   if (!allow.neg) { 
     estim = rescale( fixRight(fixLeft( estim )) , f5)  
   } # !allow.neg
  
   return(estim)
   
} # fit Beers


tmp                 = matrix(NA,A, ncol(f5), dimnames=list(age, dimnames(f5)[[2]]) )
tmp[paste(15:54),]  = apply( f5[-1,], 2, fit.Beers, allow.neg=TRUE)
OUT$Beers$HFD$unadj = round( tmp, 4)

tmp                 = matrix(NA,A, ncol(f5), dimnames=list(age, dimnames(f5)[[2]]) )
tmp[paste(15:54),]  = apply( f5[-1,], 2, fit.Beers, allow.neg=FALSE)
OUT$Beers$HFD$adj = round( tmp, 4)

tmp                 = matrix(NA,A, nrow(IDB), dimnames=list(age, dimnames(IDB)[[1]]) )
tmp[paste(15:54),]  = apply( rbind(t(IDB),0),   2, fit.Beers, allow.neg=TRUE)
OUT$Beers$IDB$unadj = round( tmp, 4)

tmp                 = matrix(NA,A, nrow(IDB), dimnames=list(age, dimnames(IDB)[[1]]) )
tmp[paste(15:54),]  = apply( rbind(t(IDB),0),   2, fit.Beers, allow.neg=FALSE)
OUT$Beers$IDB$adj = round( tmp, 4)


##############################################################################
# CS method last 
##############################################################################

## read constants
K7 = as.matrix( read.csv("K7.csv", header=T, row.names=1) )
K9 = as.matrix( read.csv("K9.csv", header=T, row.names=1) )

## construct S matrix for accumulating fine grid into single-year intervals
a = as.numeric( dimnames(K7)[[1]] )
S = outer( age, a, function(aa,xx) { 1 *((aa <= xx)&(xx < aa+1))})
S = sweep(S, 1, apply(S,1,sum), "/")
dimnames(S) = list(age, a)


tmp                 = S %*% K9 %*% f5
OUT$CS$HFD$unadj    = round( tmp, 4)
OUT$CS$HFD$adj      = pmax( round( tmp, 4), 0)

tmp                 = S %*% K7 %*% t(IDB)
OUT$CS$IDB$unadj    = round( tmp, 4)
OUT$CS$IDB$adj      = pmax( round( tmp, 4), 0)

##############################################################################
# save to dput file 
##############################################################################

dput( OUT, file=outfile)



