###################################################################################
# calculate and summarize fitting errors for the HFD schedules where 1fx values
#  are observed, and calculate some indices of "badness" for fitted IDB schedules
###################################################################################

rm(list=ls())
memory.limit(4000)

DATA = dget("fert.data.dput")
EST  = dget("fert.estimates.dput")

## RMSE by age
RMSE = function(est) {
  aa = as.numeric( dimnames(est)[[1]])  ## which ages?
  e  = est - DATA$f1[paste(aa),]        ## calc all errors
  e[DATA$f1.split[paste(aa),]] = NA     ## remove cells originally from multi-year data
  apply(e, 1, function(z) sqrt(mean(z^2,na.rm=T)) ) 
} # RMSE

## overall RMSE for an age group
RMSE.group = function(est, group=12:54) {
  e  = est[paste(group),] - DATA$f1[paste(group),]  ## calc all errors
  e[DATA$f1.split[paste(group),]] = NA              ## remove cells originally from multi-year data
  sqrt(mean(e^2,na.rm=T))   
} #RMSE.group

## overall RMSE by country
RMSE.country = function(est) {
  cname = dimnames(est)[[2]]
  country = substr( cname, 1, nchar(cname)-4 )

  e  = est - DATA$f1       ## calc all errors
  e[DATA$f1.split] = NA    ## remove cells originally from multi-year data
  
  result = sqrt( tapply( apply( e^2, 2, mean, na.rm=T), country, mean) )
  
  return(result)
} #RMSE.country

### calculate for each method at each single year of age,
###   over the HFD cells that come from one-year data
rmse.HFDsplit    = RMSE( EST$HFDsplit$HFD)
rmse.Beers.unadj = RMSE( EST$Beers$HFD$unadj)
rmse.Beers.adj   = RMSE( EST$Beers$HFD$adj)
rmse.CS.unadj    = RMSE( EST$CS$HFD$unadj)
rmse.CS.adj      = RMSE( EST$CS$HFD$adj)


## plot the errors by age (this is tricky because of all the overlapping lines)

age = 12:54

graphics.off()
png( file="compare RMSE by age.png", height=8, width=12, units="in", res=300)
  plot(age+.5, rmse.Beers.unadj, type="n", bty="l",
          xlab="Age", ylab="RMSE", ylim=c(0, .014), axes=F)
    axis(1, pos=0, at=seq(15,55,10))
    axis(2)
   
    C = rmse.CS.adj
    polygon( c(11.5, age+.5,54.5),c(0,C,0), col=grey(.70))
  
  #  lines(age+.5, rmse.Beers.unadj, pch="b", type="b", lwd=2, col=1, cex=.80)
    lines(age+.5, rmse.Beers.adj,   pch="B", type="b", lwd=6, col="seagreen")
    lines(age+.5, rmse.HFDsplit,    pch="H", type="b", lwd=6, lty=2, col="red")
  #  lines( age+.5, rmse.CS.unadj,   pch="c", type="b", lwd=2, col="purple", cex=.80)
  #  lines( age+.5, rmse.CS.adj,     pch="C", type="b", lwd=4, col="purple")
   
    abline(v=seq(10,60,5),col=grey(.6))
    text(25, .001, "Calibrated Spline", cex=1.8)
    text(23, .011, "Beers", col="seagreen", cex=1.8)
    text(30, .007, "HFD Protocol", col="red", cex=1.8)
dev.off()


########################################
## by country

r.HFD   = RMSE.country( EST$HFDsplit$HFD )
r.Beers = RMSE.country( EST$Beers$HFD$adj)
r.Beers.unadj = RMSE.country( EST$Beers$HFD$unadj)

r.CS    = RMSE.country( EST$CS$HFD$adj)
r.CS.unadj    = RMSE.country( EST$CS$HFD$unadj)

## count how many countries each method "wins" with low RMSE
table( unlist( apply( cbind(r.CS,r.CS.unadj, r.Beers,r.Beers.unadj, r.HFD), 1, which.min)) )



windows(record=T)
# matplot( r.CS,  cbind(r.HFD, r.Beers), pch=c("H","B"), col=c("red","seagreen"), 
#          cex=1.8, type="p")
# #text( r.CS,  r.HFD, names(r.HFD), cex=.70, col="red")
# 
# abline(0,1)


## select the indices of the non-missing r.CS values in descending order of r.CS
o = rev(order(r.CS))
o = o[is.finite(r.CS[o])]

op = par(no.readonly=TRUE)

png( file="compare RMSE by country.png", height=8, width=12, units="in", res=300)
  par(mai=c(1.6,.8,.8,.4))
  plot( r.CS[o], type="n", lwd=8 , axes=F, xlab="", ylab="RMSE",
           ylim=range(0,r.CS,r.HFD,r.Beers,na.rm=T))
  polygon( x= c(1,seq(o),length(o)), 
           y= c(0, r.CS[o],0), col=grey(.70) )
  points( seq(o), r.CS[o], pch="X", cex=1.20)
  segments( seq(o), 0, seq(o), r.CS[o])
  axis(2, at=seq(0,.01,.002))
#  axis(1, at=seq(o), labels=names(r.CS)[o], cex=.60, pos=0, srt=90)
  text( seq(o), 0, names(r.CS)[o], cex=1.20, srt= -45, adj=1) 
  lines( r.HFD[o], col="red",  lwd=5, lty=2, type="b", pch="H")
  lines( r.Beers[o], col="seagreen",  lwd=5, type="b", pch="B")

  text(4.5, .0015, "Calibrated Spline", cex=1.8, adj=0)
  text(7.5, .0055, "Beers", col="seagreen", cex=1.8, adj=0)
  text(4.5, .0065, "HFD Protocol", col="red", cex=1.8, adj=0)


dev.off()

par(op)



########################################

agelist = list(12:24, 25:34, 35:54, 12:54)


################# RMSE table
print("HFD")
for (j in agelist) {  print( RMSE.group( EST$HFDsplit$HFD, j) ) }

print("Beers Unadj")
for (j in agelist) {  print( RMSE.group( EST$Beers$HFD$unadj, j) ) }
print("Beers adj")
for (j in agelist) {  print( RMSE.group( EST$Beers$HFD$adj, j) ) }

print("CS Unadj")
for (j in agelist) {  print( RMSE.group( EST$CS$HFD$unadj, j) ) }
print("CS adj")
for (j in agelist) {  print( RMSE.group( EST$CS$HFD$adj, j) ) }

################# negative values
 
countneg = function(est, lim=c(0,-.0005,-.0050)) {
  for (L in lim) print( mean( est < L, na.rm=T))
}


print("HFDsplit")
countneg( cbind(EST$HFDsplit$HFD, EST$HFDsplit$IDB))

print("Beers unadj")
countneg( cbind(EST$Beers$HFD$unadj,EST$Beers$IDB$unadj))
print("Beers adj")
countneg( cbind(EST$Beers$HFD$adj,EST$Beers$IDB$adj))

print("CS unadj")
countneg( cbind(EST$CS$HFD$unadj,EST$CS$HFD$unadj))
print("CS adj")
countneg( cbind(EST$CS$HFD$adj,EST$CS$HFD$adj))



####### average second differences (roughness) over HFD (g=9) and IDB (g=7)

# mean sq second diff
mssd = function(x) {
  tmp = diff(diff(x))
  mean(tmp^2, na.rm=TRUE)
}

## -- g=9
#Beers
yhat = cbind( EST$Beers$HFD$adj)[paste(15:49),]
sqrt( mean( apply(yhat, 2, mssd)))

yhat = cbind( EST$Beers$HFD$unadj)[paste(15:49),]
sqrt( mean( apply(yhat, 2, mssd)))

#HFD1
yhat = cbind( EST$HFDsplit$HFD)[paste(15:49),]
sqrt( mean( apply(yhat, 2, mssd)))

#CS
yhat = cbind( EST$CS$HFD$unadj)[paste(15:49),]
sqrt( mean( apply(yhat, 2, mssd)))

yhat = cbind( EST$CS$HFD$adj)[paste(15:49),]
sqrt( mean( apply(yhat, 2, mssd)))

## -- g=7

#Beers
yhat = cbind(  EST$Beers$IDB$adj)[paste(15:49),]
sqrt( mean( apply(yhat, 2, mssd)))

yhat = cbind(  EST$Beers$IDB$unadj)[paste(15:49),]
sqrt( mean( apply(yhat, 2, mssd)))

#HFD1
yhat = cbind(  EST$HFDsplit$IDB)[paste(15:49),]
sqrt( mean( apply(yhat, 2, mssd)))

#CS
yhat = cbind(  EST$CS$IDB$unadj)[paste(15:49),]
sqrt( mean( apply(yhat, 2, mssd)))

yhat = cbind(  EST$CS$IDB$adj)[paste(15:49),]
sqrt( mean( apply(yhat, 2, mssd)))

####### multiple local maxima

nmax = function(y) { 
   tmp = y[!is.na(y)]     # remove NA
   tmp = sign(diff(tmp))  # signs of slopes
   tmp = tmp[tmp !=0]     # remove any plateaus
   sum( diff(tmp) < 0) 
}

multimodal = function(est) {
  mean( apply(est, 2, nmax) > 1)
}

print("HFDsplit")
multimodal( cbind(EST$HFDsplit$HFD, EST$HFDsplit$IDB))

print("Beers unadj")
multimodal( cbind(EST$Beers$HFD$unadj,EST$Beers$IDB$unadj))
print("Beers adj")
multimodal( cbind(EST$Beers$HFD$adj,EST$Beers$IDB$adj))

print("CS unadj")
multimodal( cbind(EST$CS$HFD$unadj,EST$CS$HFD$unadj))
print("CS adj")
multimodal( cbind(EST$CS$HFD$adj,EST$CS$HFD$adj))


