################################################################
#  Fit a spline Bw over ages x1...xN to minimize wtd sum of penalties
# (1)  FIT:    small eps = f - GBw
# (2)  SHAPE:  small v = MSBw  (M is projection matrix for HFD comp.)
#
# calculate Kf,Kw, and moment multipliers and write to text files
################################################################

rm(list=ls())
graphics.off()
windows(record=T) 
op = par(no.readonly=T)  # save graphics defaults for later resets

beers.hue = "royalblue"

library(splines)

PC        = 3         # number of principal SVD components
Wstar     = 2000      # number of women per single-year age

splineDegree = 2      # decided on 2 rather than 3 because HFD fits are SLIGHTLY better

#----------------------------------------
# read prepared f5,f1,SVD,NMF... data
#----------------------------------------
D = dget("fert.data.dput")
for (k in names(D)) assign(k, D[[k]])

big = cbind(f1, IDB1)

col.name = dimnames(big)[[2]]
country = substr(col.name, 1, nchar(col.name)-4)
year    = as.numeric( substr(col.name, nchar(col.name)-3,nchar(col.name)) )

country[country==""] <- "IDB"      # for non-HFD schedules
country.list = unique(country)   

alpha = 12
beta  = 55

age   = alpha:(beta-1)
A     = length(age)

#--------------------------------------
# set up grid and basis functions
#--------------------------------------

na    = 2
Delta = 1/na

a        = seq(alpha+Delta/2, beta-Delta/2, Delta)
y        = NA*a
names(y) = a
N        = length(y)

#knot = seq(alpha, beta, 2 )  
knot = seq(alpha, beta, 2 )  

## quad spline basis
B = bs(a, knots=knot, degree= splineDegree)
keep = (splineDegree+1):(ncol(B)-((splineDegree+1):0))
B   = B[,keep]


nparam = ncol(B)

dimnames(B)[[1]] = a

matplot(a, B, type="l", lty=1, lwd=6)

##single year matrix
S = outer(age, a, function(aa,xx) { 1 * (aa <= xx) * (xx < aa+1)  })
dimnames(S) = list(age,a)
S = sweep(S,1,apply(S,1,sum),"/")

## age grouping matrix for fine grid
L           = seq(10,50,5)
G           = outer(L,a, function(LL,xx) { 1 * (LL <= xx) * (xx < LL+5)  })
dimnames(G) = list(L,a)
G           = sweep(G,1,apply(G,1,sum),"/")

## age grouping matrix for single-year grid
Grouper           = outer(L,age, function(LL,xx) { 1 * (LL <= xx) * (xx < LL+5)  })
dimnames(Grouper) = list(L,age)
Grouper           = sweep(Grouper,1,apply(Grouper,1,sum),"/")

###########################################################

#--------------------------------------
# identify whether columns of big have
# complete 1-year schedules 
#--------------------------------------

complete.cols      = c( apply(f1.split, 2, function(x) {all(!x)} ) ,
                         rep(FALSE,nrow(IDB)))
complete.countries = unique( country[complete.cols])

#--------------------------------------
# for each country with at least one
#  complete 1-year schedule...
#  (1) remove that country's data from "big"
#  (2) recalculate the principal components using
#        only the data from *other* countries
#  (3) calculate Kf and the fitted schedules for
#        the country's 1-year schedules
#  (4) calculate and save the fitting errors fhat1 - f1
#--------------------------------------

## read previously calculated Beers estimates for
##  ages 15..54  (note: not 12,13,14)

beers.f1hat = dget("Beers f1 estimates.dput")$HFD

rmse = array(NA, c(A, length(complete.countries),2),
                 dimnames=list(age,complete.countries,c("spline","beers")))


for (co in complete.countries) {
  # remember that co is a string 
  
  own = (country == co)
  tmp = big[paste(age),!own]
  
  U   = svd(tmp)$u[,1:PC]
  dimnames(U) = list(age,NULL)

  PR  = U %*% solve(t(U) %*% U) %*% t(U)  # projection
  M   = diag(A) - PR                      # residuals
  
  v   = M %*% f1[paste(age),]
  tot = matrix(0,A,A)
  for (i in 1:ncol(v)) {
   z = matrix(v[,i], ncol=1)
   tot = tot + 1/ncol(v) * (z %*% t(z))
  }
  
  Vsh    = tot
  
  # add constant to diagonal to stabilize empirical estimate before inverting
  VshInv = solve( Vsh + 0.1 * median(diag(Vsh))*diag(A))    

  VfitInv = 5*Wstar/(.10) * diag(length(L))
  
  C1 =       t(B) %*% t(G) %*% VfitInv %*% G %*% B
  C1 = C1 +  t(B) %*% t(S) %*% t(M)  %*% VshInv %*% M  %*% S %*% B  
    
  C2 = t(B) %*% t(G) %*% VfitInv
  
  Kw = solve(C1) %*% C2
  dimnames(Kw) = list(NULL, L)
  
  Kf    = B %*% Kw
  
  ## plot Kf once (it will be slightly different for every selected country)
  if (co == complete.countries[1]) {
    matplot(a, Kf, type="l", lty=1, lwd=3)
   abline(v=seq(10,60,5),col=grey(.60),lty=2)
  }
  
  ## calculate fits and errors for the complete 1-yr schedules in the country of interest
  ff1 = big[, own & complete.cols]
  yrs = paste( range( year[own & complete.cols]), collapse="-")
  
  ff5 = Grouper %*% ff1
  
  fhat1 = S %*% Kf %*% ff5
  
  eps = fhat1 - ff1   # fitting errors

   rmse[,co,"spline"] = apply(eps, 1, function(z) { sqrt(mean(z^2)) })
  
  
  beers.cols = match( dimnames(ff1)[[2]] , dimnames(beers.f1hat)[[2]] )
  berr = beers.f1hat[,beers.cols] - ff1[-(1:3),]
   rmse[-(1:3),co,"beers"] = apply(berr, 1, function(z) { sqrt(mean(z^2)) })
  
  plot(age+.5, rmse[,co,"spline"], type="o", pch=16, 
        main=paste(co,yrs), ylim=c(0,.015), bty="l", ylab="RMSE",lwd=4,col=2 )
  lines(age[-(1:3)]+.5, rmse[-(1:3),co,"beers"], type="o", pch=1, col=beers.hue,lwd=2) 
  abline(h=0,lwd=2)
  abline(v=seq(10,60,5),h=seq(0,.015,.005),lty=2,col=grey(.50))
  
  
  
## show an example plot: say GBRTENW2009
  if (co == "GBRTENW") {
    sel = "GBRTENW2009"
    png('spline and beers fit Engl Wales 2009.png', width=11, height=7, units="in", res=300)
      par( mfrow=c(1,1), mai=c(1,.8,.8,.3), bty="l")
      
      plot(age+.5, ff1[,sel], type="n", bty="l", xlab="Age", ylab="ASFR", 
             main="England & Wales 2009")
    
      lines( a, Kf %*% ff5[,sel], col=2, lwd=8)
      lines( age[-(1:3)]+.5, beers.f1hat[,sel], col=beers.hue, lwd=5, lty=4)
      points(age+.5, ff1[,sel], pch=16, cex=1.3)
    
      legend(45,.10,c("Spline","Beers","1fx"), lty=c(1,4,NA), col=c("red",beers.hue,"black"),
                lwd=c(8,4,NA), pch=c(NA,NA,16), bty="n")
    
      segments(L,0,L,ff5[,sel],col=grey(.60))
      segments(L,ff5[,sel],L+5,ff5[,sel],col=grey(.60))
      segments(L+5,ff5[,sel],L+5,0,col=grey(.60))
      segments(L,0,L+5,0,col=grey(.60))
      par(op)
    dev.off()
  } # if co

  ## show an example plot: say SWE1900
  if (co == "SWE") {
    sel = "SWE1900"
    plot(age+.5, ff1[,sel], type="n", bty="l", xlab="Age", ylab="ASFR", 
         main="Sweden 1900")
    lines( age+.5, fhat1[,sel], col=2, lwd=4)
    lines( age[-(1:3)]+.5, beers.f1hat[,sel], col=beers.hue, lwd=2, lty=1)
    points(age+.5, ff1[,sel], pch=16, cex=1.3)
    segments(L,0,L,ff5[,sel],col=grey(.60))
    segments(L,ff5[,sel],L+5,ff5[,sel],col=grey(.60))
    segments(L+5,ff5[,sel],L+5,0,col=grey(.60))
    segments(L,0,L+5,0,col=grey(.60))
    
  } # if co

  ## show an example plot: say AUT1980
  if (co == "AUT") {
    sel = "AUT1980"
    plot(age+.5, ff1[,sel], type="n", bty="l", xlab="Age", ylab="ASFR", 
         main="Austria 1980")
    lines( age+.5, fhat1[,sel], col=2, lwd=4)
    lines( age[-(1:3)]+.5, beers.f1hat[,sel], col=beers.hue, lwd=2, lty=1)
    points(age+.5, ff1[,sel], pch=16, cex=1.3)
    segments(L,0,L,ff5[,sel],col=grey(.60))
    segments(L,ff5[,sel],L+5,ff5[,sel],col=grey(.60))
    segments(L+5,ff5[,sel],L+5,0,col=grey(.60))
    segments(L,0,L+5,0,col=grey(.60))
    
  } # if co
  
} # for co


rmse

png('rmse by age.png', width=8, height=8, units="in", res=300)
  par( mfrow=c(1,1), mai=c(1,.8,.8,.3), bty="l")
  
  matplot(age+.5, apply( rmse, c(1,3), mean, na.rm=T),bty="l", 
            pch=c(16,1),type="o",lwd=c(5,2),xlab="Age",ylab="RMSE",
            lty=1,col=c("red",beers.hue),cex=1.5, axes=F)
  abline(v=seq(10,60,5),h=seq(.002,.008,.002),lty=1,col=grey(.70))
  
  axis(1,pos=0, at=seq(10,50,10))
  axis(2,pos=10)
  
  text(19,.0015,"Spline",cex=2,col="red")
  text(15,.0062,"Beers",cex=2,col=beers.hue)
  par(op)
dev.off()

par(mfrow=c(1,2))


xy = sqrt(apply(rmse^2,c(1,3),mean, na.rm=T ))
plot(xy[,1], xy[,2], xlim=range(xy,0,na.rm=T), ylim=range(xy,0,na.rm=T), type="n",
     bty="l",xlab="Spline RMSE", ylab="Beers RMSE", main="RMSE by Age")
text(xy[,1], xy[,2], dimnames(rmse)[[1]], cex=1, adj=0)
abline(0,1,lwd=2)

xy = sqrt( apply(rmse^2,c(2,3),mean, na.rm=T ) )
plot(xy[,1], xy[,2], xlim=range(xy,0), ylim=range(xy,0), pch=16,
       bty="l",xlab="Spline RMSE", ylab="Beers RMSE", main="RMSE by Country")
text(xy[,1]+.7e-4, xy[,2], dimnames(rmse)[[2]], cex=1, adj=0)
abline(0,1,lwd=2)


par(op)

png('rmse by country.png', width=8, height=8, units="in", res=300)
  par( mfrow=c(1,1), mai=c(1,.8,.8,.3), bty="l")
  
  xy = sqrt( apply(rmse^2,c(2,3),mean, na.rm=T ) )
  plot(xy[,1], xy[,2], xlim=c(0,0.005), ylim=c(0,.005), pch=16,
       bty="l",xlab="Spline RMSE", ylab="Beers RMSE", axes=F)
  axis(1,pos=0)
  axis(2,pos=0)
  text(xy[,1]+.7e-4, xy[,2], dimnames(rmse)[[2]], cex=1, adj=0)
  segments(0,0,.005,.005,lwd=2)
  
  par(op)
dev.off()
