################################################################################
##
## Subversion file information - please do not edit this block
## $Rev: 852 $
## $Date: 2010-11-22 14:46:34 +0100 (Mon, 22 Nov 2010) $
## $Author: nash $
## $Id: indbExposure.R 852 2010-11-22 13:46:34Z nash $
##
################################################################################

indbExposure <-
function(pop, monthlyBirths = NULL, tAdj = NULL,
           outfile = NULL, countryCode = "XXX", verbose = TRUE) {
## Calculate female population exposure, based on cohort monthly birth counts
## where they are available
##
## @inputs
## - popFile is the path to the HMD Lexis-DB file of female population counts
##   and deaths, or a data frame representing this
## - monthlyBirths is the path to the file of birth counts by month in the
##   standard HFD format, or a data frame representing this
##   - if NULL then no monthly birth counts will be used in the calculation
## - tAdj is the path to the HMD territorial adjustment file, in the case
##   that a territorial adjustment is to be made, or a data frame representing this
##   - if NULL then no territorial adjustment is made
## - outfile is the path to the output file to be created
##   - if NULL then a file XXXexp.txt in the working directory, with XXX being
##     the country code. An attempt is made to deduce this from the names
##     of other files or from the data if not specified
##   - if NA then no output file is created
## - countryCode is the code of the country, used for automatically generating
##   an output filename if necessary
## - verbose is whether status will be reported
## @returns
## - the female population exposures as a data frame with the columns
##   Year,Age,Triangle,Cohort,Exposure

  # Population counts and deaths by triangle (HMD Lexis-DB file)
  popNames <- c("Year", "Age", "Triangle", "Cohort", "Population", "Deaths")
  if (!is.data.frame(pop)) {
    if (countryCode == "XXX") countryCode <- toupper(strsplit(basename(pop), "(f|\\.)")[[1]][2])
    pop <- read.csv(pop, header = FALSE, na.strings = ".")
    names(pop) <- popNames
  } else {
    stopifnot(names(pop) == popNames)
  }

  # Territorial adjustment factors, if necessary (only Vx for females needed)
  tAdj <-
    if (!is.null(tAdj)) {
      if (!is.data.frame(tAdj)) {
        if (countryCode == "XXX") countryCode <- toupper(strsplit(basename(tAdj), "tadj", fixed = TRUE)[[1]][1])
        tAdj <- read.csv(tAdj, na.strings = ".")
      }
      stopifnot(all(c("PopName", "Year", "Sex", "Type", "Value") %in% names(tAdj)))
      if (countryCode == "XXX") countryCode <- toupper(tAdj$PopName[1])
      tAdj <- tAdj[tAdj$Sex == "f" & tAdj$Type == "Vx", ]
    } else {
      NULL
    }

  # birth characteristics
  if (!is.null(monthlyBirths)) {
    if (!is.data.frame(monthlyBirths)) {
      if (countryCode == "XXX") countryCode <- toupper(strsplit(basename(monthlyBirths), "monthly", fixed = TRUE)[[1]][1]) 
      monthlyBirths <- read.csv(monthlyBirths, na.strings = ".")
    }
    stopifnot(all(c("PopName", "Year", "Month", "Births", "LDB") %in% names(monthlyBirths)))
    monthlyBirths <- monthlyBirths[monthlyBirths$LDB == 1, ]
    monthlyBirths[is.na(monthlyBirths$Year), "Year"] <- monthlyBirths[is.na(monthlyBirths$Year), "YearReg"]
    if (countryCode == "XXX") countryCode <- toupper(monthlyBirths$PopName[1])
    # we calculate totals ourselves and ignore unknowns for the minute
    monthlyBirths <- monthlyBirths[monthlyBirths$Month %in% 1:12, c("Year", "Month", "Births")]
    monthlyBirths$Month <- as.numeric(levels(monthlyBirths$Month)[monthlyBirths$Month])
    # it can also be the case that the monthly births are NA, so discard those rows
    monthlyBirths <- monthlyBirths[!is.na(monthlyBirths$Births), ]
    # finally change year of registration to year of birth where necessary
    monthlyBirths$Year[is.na(monthlyBirths$Year)] <- monthlyBirths$YearReg[is.na(monthlyBirths$Year)]
  }
  # calculate mean date of birth for each cohort, if we have monthly births data
  m <- if(!is.null(monthlyBirths)) {
         by(monthlyBirths,
            monthlyBirths$Year,
            function(year) {
              result <- list()
              # work out if the year is a leap year, so we can adjust accordingly
              y <- year[1, "Year"]
              isLeap <- as.Date(sprintf("%4.0f-03-01", y)) - as.Date(sprintf("%4.0f-02-01", y)) > 28
              # average time from start of year for each month, in days
              cumTime <- c(15.5, 45, 74.5, 105, 135.5, 166,
                           196.5, 227.5, 258, 288.5, 319, 349.5)
              # leap year if more than 28 days between 1st Feb and 1st Mar!
              if (isLeap) {
                cumTime[2] <- cumTime[2] + 0.5 # mean date in February increased by 0.5 days
                cumTime[3:12] <- cumTime[3:12] + 1 # mean date after February increased by 1
              }
              # convert days to years
              cumTime <- cumTime / if (isLeap) 366 else 365
              # mean date of birth in years from start of year
              return(weighted.mean(cumTime, year$Births))
            })
          } else {
            list()
          }

  years <- unique(pop$Year)
  years <- years[1:(length(years) - 1)]
  ages <- unique(pop$Age)
  agesc <- as.character(ages)
  cohorts <- (min(years) - max(ages) - 1):(max(years)-min(ages))

  E <- pop[pop$Year %in% years, 1:5]
  names(E)[5] <- "Exposure"

  # binary masks for triangles, to avoid recalculating
  popTriangle1 <- pop$Triangle == 1
  popTriangle2 <- pop$Triangle == 2

  if (verbose) {
    if (Sys.info()["sysname"] == "Windows") {
      progbar <- winProgressBar
      upprog <- setWinProgressBar
    } else if (exists("tkProgressBar", mode = "function")) {
      progbar <- tkProgressBar 
      upprog <- setTkProgressBar
    } else {
      progbar <- txtProgressBar
      upprog <- setTxtProgressBar
    }
    pb <- progbar(min = min(years), max = max(years), initial = min(years), title = paste("Calculating exposures for", countryCode), label = paste("Processing year: ", min(years)))
    on.exit(close(pb), add = TRUE)
  }

  # TODO: referencing on data frames is very slow, investigate ways to speed this up
  for (n in years) {
    if (verbose) upprog(pb, n, label = paste("Processing year: ", n))
    nc <- as.character(n)
    # binary masks for years
    popYearN <- pop$Year == n
    popYearNp1 <- pop$Year == n + 1
    for (i in ages) {
      ic <- as.character(i)
      # binary masks for age and then triangles which may be referenced more than once
      popAgeI <- pop$Age == i
      t1 <- popAgeI & popYearN & popTriangle1
      t2 <- popAgeI & popYearN & popTriangle2
      Pni <- pop[t2, 5] # P(n)(i)
      Pnp1i <- pop[popAgeI & popYearNp1 & popTriangle2, 5] # P(n+1)(i)
      if (Pni == -1 || Pnp1i == -1) {
        # values of -1 indicate missing data
        E[t1, 5] <- NA
        E[t2, 5] <- NA
      } else {
        if ((!is.null(tAdj)) & length(Vx <- tAdj[tAdj$Year == n + 1 & tAdj$Age == i, "Value"]) == 1) {
          Pnp1i <- Pnp1i / Vx
        }
        DL <- pop[t1, 6] # lower triangle deaths
        DU <- pop[t2, 6] # upper triangle deaths
        if (length(Pnp1i) == 0) {
          # this probably should never actually occur, but TODO: work out if there is an intelligent solution
          stop(sprintf("Population count data does not cover a sufficient timespan (year %4.0f missing)", n + 1))
        }
        # if monthly birth data not available for cohort then mean date of birth is assumed to be 0.5
        m2 <- if ((n - i) %in% names(m)) m[[as.character(n - i)]] else 0.5
        m1 <- if ((n - i - 1) %in% names(m)) m[[as.character(n - i - 1)]] else 0.5
        E[t1, 5] <- if ((n - i) %in% names(m)) Pnp1i * (1 - m2) else (Pnp1i / 2) + (DL / 6)
        E[t2, 5] <- if ((n - i - 1) %in% names(m)) Pni * m1 else (Pni / 2) - (DU / 6)
      }
    }
  }

  # trim any rows that are missing data
  E <- E[!is.na(E[, 5]), ]
  
  if (is.null(outfile)) outfile <- paste(countryCode, "exp.txt", sep = "")
  if (verbose && !is.na(outfile)) upprog(pb, n, label = paste("Writing", outfile))  
  if (!is.na(outfile)) write.csv(E, outfile, row.names = FALSE, quote = FALSE)
  
  return(E)
}

